@@ -1,7 +1,7 @@
 use crate::args::ArgSpec;
 use crate::{arg, arg_enum, args};
-use cdx::check::{make_match, BufCheck, CheckOr};
-use cdx::column::{ColumnCount, ColumnLiteral, ColumnWhole, Writer};
+use cdx::column::{ColumnCount, ColumnHeader, ColumnLiteral, ColumnWhole, Writer};
+use cdx::matcher::{Match, MatchMaker, MultiMatcher, MultiMode};
 use cdx::text::Text;
 use cdx::{
     err, get_reader, get_writer, Error, HeaderChecker, HeaderMode, Reader, Result, HEADER_MODE,
@@ -90,8 +90,8 @@ pub fn main(argv: &[String]) -> Result<()> {
     let mut checker = HeaderChecker::new();
     let mut pad = PadMode::All;
     let mut num = LineNumber::new();
-    let mut skips = CheckOr::new();
-    let mut removes = CheckOr::new();
+    let mut skips = MultiMatcher::new(MultiMode::Or);
+    let mut removes = MultiMatcher::new(MultiMode::Or);
 
     for x in args {
         if x.name == "header" {
@@ -113,14 +113,15 @@ pub fn main(argv: &[String]) -> Result<()> {
         } else if x.name == "end" {
             num.set("number,1,end")?;
         } else if x.name == "skip" {
-            skips.push(make_match(&x.value)?);
+            skips.push(MatchMaker::make(&x.value)?);
         } else if x.name == "remove" {
-            removes.push(make_match(&x.value)?);
+            removes.push(MatchMaker::make(&x.value)?);
         } else {
             unreachable!();
         }
     }
 
+    let mut header = ColumnHeader::new();
     let mut w = get_writer("-")?;
     let slow = num.do_it || !skips.is_empty() || !removes.is_empty();
 
@@ -142,7 +143,7 @@ pub fn main(argv: &[String]) -> Result<()> {
         if num.do_it && num.end {
             not_v.push(Box::new(ColumnLiteral::new(b"", "unused")));
         }
-        let mut not_header: Vec<u8> = Vec::new();
+        let mut not_header = String::new();
 
         for x in &files {
             let mut f = Reader::new();
@@ -151,20 +152,22 @@ pub fn main(argv: &[String]) -> Result<()> {
                 continue;
             }
             v.lookup(&f.names())?;
+            header.clear();
             not_header.clear();
+            v.add_names(&mut header, f.header())?;
             if f.cont.has_header {
-                v.write_names(&mut not_header, f.header())?;
+                not_header = header.get_head(b'\t');
             }
-            if checker.check(&not_header, x)? {
-                w.write_all(&not_header)?;
+            if checker.check(not_header.as_bytes(), x)? {
+                w.write_all(not_header.as_bytes())?;
             }
             if f.is_done() {
                 return Ok(());
             }
             f.do_split = false;
             loop {
-                if !removes.ucheck(f.curr_nl()) {
-                    if skips.ucheck(f.curr_nl()) {
+                if !removes.umatch(f.curr_nl()) {
+                    if skips.umatch(f.curr_nl()) {
                         not_v.write(&mut w, f.curr())?;
                     } else {
                         v.write(&mut w, f.curr())?;
