@@ -1,24 +1,40 @@
 use crate::args::ArgSpec;
 use crate::{arg, arg_enum, args};
-use cdx::check::*;
+use cdx::expr;
+use cdx::matcher::*;
 use cdx::{get_writer, Error, HeaderChecker, HeaderMode, LookbackReader, Result, HEADER_MODE};
 use std::str::FromStr;
 
 pub fn main(argv: &[String]) -> Result<()> {
     let prog = args::ProgSpec::new("Select uniq lines.", args::FileCount::Many);
-    const A: [ArgSpec; 2] = [
+    const A: [ArgSpec; 6] = [
         arg! {"pattern", "p", "Col,Spec,Pattern", "Select line where this col matches this pattern."},
+        arg! {"show-matchers", "s", "", "Print available matchers"},
+        arg! {"show-const", "", "", "Print available constants"},
+        arg! {"show-func", "", "", "Print available functions"},
+        arg! {"or", "o", "", "A line matches if any of the matchers matches."},
         arg_enum! {"header", "h", "Mode", "header requirements", &HEADER_MODE},
     ];
     let (args, files) = args::parse(&prog, &A, argv);
 
     let mut checker = HeaderChecker::new();
-    let mut list = CheckList::new();
+    let mut list = MultiLineMatcher::new(MultiMode::And);
     for x in args {
         if x.name == "header" {
             checker.mode = HeaderMode::from_str(&x.value)?;
         } else if x.name == "pattern" {
-            list.push(Box::new(ColChecker::new(&x.value)?));
+            list.push_spec(&x.value)?;
+        } else if x.name == "show-matchers" {
+            MatchMaker::help();
+            return Ok(());
+        } else if x.name == "or" {
+            list.multi = MultiMode::Or;
+        } else if x.name == "show-const" {
+            expr::show_const();
+            return Ok(());
+        } else if x.name == "show-func" {
+            expr::show_func();
+            return Ok(());
         } else {
             unreachable!();
         }
@@ -62,7 +78,7 @@ pub fn main(argv: &[String]) -> Result<()> {
         } else {
             let mut fails = 0;
             loop {
-                if !list.ok_verbose(f.curr_line()) {
+                if !list.ok_verbose(f.curr_line(), false) {
                     fails += 1;
                     if fails >= max_fails {
                         break;
