@@ -28,6 +28,7 @@
     clippy::cargo,
 )]
 #![allow(clippy::multiple_crate_versions)]
+#![allow(clippy::redundant_pub_crate)]
 
 use flate2::read::MultiGzDecoder;
 use fs_err as fs;
@@ -40,14 +41,18 @@ use std::os::unix::ffi::OsStrExt;
 use std::path::Path;
 use std::{fmt, str};
 
+pub mod agg;
 pub mod binsearch;
-pub mod check;
 pub mod column;
 pub mod comp;
 pub mod expr;
 pub mod join;
+pub mod matcher;
+pub mod num;
+mod shunting_yard;
 pub mod sort;
 pub mod text;
+mod tokenizer;
 pub mod tooltest;
 
 use crate::text::Text;
@@ -123,7 +128,7 @@ err_type!(std::num::ParseFloatError, Error::ParseFloatError);
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            Error::Error(s) => write!(f, "Cdx Error : {}", s)?,
+            Error::Error(s) => write!(f, "{}", s)?,
             Error::ParseIntError(s) => write!(f, "ParseIntError : {}", s)?,
             Error::ParseFloatError(s) => write!(f, "ParseFloatError : {}", s)?,
             Error::IoError(s) => write!(f, "IoError : {}", s)?,
@@ -146,6 +151,20 @@ pub struct FakeSlice {
     begin: u32,
     end: u32,
 }
+impl FakeSlice {
+    /// get slice from FakeSlice
+    pub fn get<'a>(&self, data : &'a [u8]) -> &'a[u8] {
+	&data[self.begin as usize..self.end as usize]
+    }
+    /// len
+    pub const fn len(&self) -> usize {
+	(self.end-self.begin) as usize
+    }
+    /// is empty?
+    pub const fn is_empty(&self) -> bool {
+	self.begin == self.end
+    }
+}
 
 /// A line of a text file, broken into fields.
 /// Access to the `lines` and `parts` is allowed, but should seldom be necessary
@@ -224,6 +243,13 @@ pub fn write_all_nl(w: &mut impl Write, buf: &[u8]) -> Result<()> {
 }
 
 impl TextLine {
+    /// assign TextLine into existing TextLine, avoiding allocation if possible
+    pub fn assign(&mut self, x: &Self) {
+        self.line.clear();
+        self.line.extend_from_slice(&x.line[..]);
+        self.parts.clear();
+        self.parts.extend_from_slice(&x.parts[..]);
+    }
     /// make a new TextLine
     pub const fn new() -> Self {
         Self {
@@ -290,10 +316,11 @@ impl TextLine {
             end += 1;
         }
         if begin != end {
-            self.parts.push(FakeSlice {
-                begin,
-                end: end - 1,
-            });
+	    let mut f = FakeSlice{begin,end};
+	    if self.line[(end-1) as usize] == b'\n' {
+		f.end -= 1;
+	    }
+            self.parts.push(f);
         }
     }
     /// return all parts as a vector
@@ -624,6 +651,22 @@ pub struct InfileContext {
     pub has_header: bool,
 }
 
+/// create appropriate header from first line of file
+pub fn make_header(line: &[u8]) -> StringLine {
+    let mut s = StringLine::new();
+    if is_cdx(line) {
+        s.line = String::from_utf8_lossy(&line[5..]).to_string();
+    } else {
+        s.line = String::new();
+        for x in 1..=line.split(|ch| *ch == b'\t').count() {
+            s.line.push_str(&format!("c{}\t", x));
+        }
+        s.line.pop();
+    }
+    s.split(b'\t');
+    s
+}
+
 // FIXME -- specify delimiter
 // if CDX and specified and different, then strip header
 /// Reader header line, if any, and first line of text
@@ -712,6 +755,10 @@ impl Reader {
     pub const fn curr(&self) -> &TextLine {
         &self.line
     }
+    /// get current line
+    pub const fn curr_line(&self) -> &TextLine {
+        &self.line
+    }
     /// get current line contents, without the trailing newline
     pub fn curr_nl(&self) -> &[u8] {
         &self.line.line[0..self.line.line.len() - 1]
@@ -720,6 +767,14 @@ impl Reader {
     pub const fn header(&self) -> &StringLine {
         &self.cont.header
     }
+    /// get header
+    pub const fn header_line(&self) -> &String {
+        &self.cont.header.line
+    }
+    /// has header
+    pub const fn has_header(&self) -> bool {
+        self.cont.has_header
+    }
     /// get column names
     pub fn names(&self) -> Vec<&str> {
         self.cont.header.vec()
@@ -733,6 +788,10 @@ impl Reader {
     pub const fn is_empty(&self) -> bool {
         self.cont.is_empty
     }
+    /// get delimiter
+    pub const fn delim(&self) -> u8 {
+        self.cont.delim
+    }
     /// have we read all the lines?
     pub const fn is_done(&self) -> bool {
         self.cont.is_done
@@ -786,6 +845,10 @@ impl LookbackReader {
             curr: 0,
         }
     }
+    /// get delimiter
+    pub const fn delim(&self) -> u8 {
+        self.cont.delim
+    }
     /// get column names
     pub fn names(&self) -> Vec<&str> {
         self.cont.header.vec()
@@ -796,7 +859,7 @@ impl LookbackReader {
         self.cont.read_header(&mut *self.file, &mut self.lines[0])
     }
     /// The full text of the header, without the trailing newline
-    pub const fn header(&self) -> &String {
+    pub const fn header_line(&self) -> &String {
         &self.cont.header.line
     }
     /// was file zero bytes?
@@ -841,14 +904,28 @@ impl LookbackReader {
         w.write_all(&self.curr_line().line)?;
         Ok(())
     }
+    /// write previous text line with newline
+    pub fn write_prev(&self, w: &mut impl Write, lookback: usize) -> Result<()> {
+        w.write_all(&self.prev_line(lookback).line)?;
+        Ok(())
+    }
     /// write header
     pub fn write_header(&self, w: &mut impl Write) -> Result<()> {
         w.write_all(self.cont.header.line.as_bytes())?;
         Ok(())
     }
+    /// write header
+    pub const fn header(&self) -> &StringLine {
+        &self.cont.header
+    }
+    /// write header
+    pub const fn has_header(&self) -> bool {
+        self.cont.has_header
+    }
 }
 
-fn prerr(data: &[&[u8]]) {
+/// print a bunch of u8 to stderr
+pub fn prerr(data: &[&[u8]]) {
     for x in data {
         std::io::stderr().write_all(x).unwrap();
     }
