@@ -1,7 +1,8 @@
 //! parse and run tooltest files
-use crate::comp::{skip_leading_white, str_to_i_whole};
-use crate::prerr;
-use crate::{err, get_reader, get_writer, Error, Infile, Result};
+use crate::matcher::{MatchMaker, Matcher};
+use crate::num::str_to_i_whole;
+use crate::text::Text;
+use crate::{err, get_reader, get_writer, prerr, Error, Infile, Result};
 //use fs_err as fs;
 use fs_err as fs;
 use std::ffi::OsStr;
@@ -11,28 +12,71 @@ use std::path::Path;
 use std::process::Command;
 use tempdir::TempDir;
 
+/// print a description of the test file format to stdout
+pub fn show_format() {
+    println!("#command CMD : the command to be run");
+    println!("#stdin : followed by the input to the program. Default empty.");
+    println!("#stdout : followed by the expected stdout output. Default empty.");
+    println!("#stderr : followed by the expected stderr output. Default empty.");
+    println!("#infile Filename : followed by the contents of the named file. '#infile foo' means that the file $TMP/foo is available to the command.");
+    println!("#outfile Filename : followed by the contents of the named file. '#outfile foo' means that the file $TMP/foo must be created by the command, with exactly that contents.");
+    println!("#nonewline : must follow the contents of one of the above four. Removes the final newline from the input or expected output.");
+    println!("#status Number : the expected exit status. Default zero.");
+    println!("'# ' : a line starting with # and a space is a comment, and is ignored");
+}
+
 #[derive(Debug, Clone, Default)]
-struct OneFile {
+struct InFile {
     name: String,
     content: Vec<u8>,
 }
 
+#[allow(missing_debug_implementations)]
+struct OutFile {
+    name: String,
+    content: Vec<u8>,
+    matcher: Matcher,
+}
+impl Default for OutFile {
+    fn default() -> Self {
+        Self {
+            name: String::new(),
+            content: Vec::new(),
+            matcher: MatchMaker::make("empty").unwrap(),
+        }
+    }
+}
+
 /// One test to be run
-#[derive(Debug, Clone, Default)]
+#[allow(missing_debug_implementations)]
+#[derive(Default)]
 pub struct Test {
     name: String,
     cmd: Vec<u8>,
     stdin: Vec<u8>,
-    stdout: Vec<u8>,
-    stderr: Vec<u8>,
+    stdout: OutFile,
+    stderr: OutFile,
     code: i32,
-    in_files: Vec<OneFile>,
-    out_files: Vec<OneFile>,
+    in_files: Vec<InFile>,
+    out_files: Vec<OutFile>,
 }
 
-/// remove item from Vec
-fn nuke(v: &mut Vec<String>, s: &str) {
+/// delete file from dir
+fn delete_file(s: &str, dir: &str) -> Result<()> {
+    let mut f = dir.to_string();
+    f.push_str(s);
+    std::fs::remove_file(f)?;
+    Ok(())
+}
+
+/// remove item from Vec and delete file
+fn nuke(v: &mut Vec<String>, s: &str, dir: &str, keep_files: bool) -> Result<()> {
     v.retain(|x| x != s);
+    if keep_files {
+        Ok(())
+    } else {
+        delete_file(s, dir)
+    }
 }
 
 /// return directory as vec of file names
@@ -90,6 +134,40 @@ impl Test {
     pub fn new() -> Self {
         Self::default()
     }
+    fn add_outfile(
+        &mut self,
+        tag: &str,
+        line: &mut Vec<u8>,
+        reader: &mut Infile,
+        need_read: &mut bool,
+    ) -> Result<bool> {
+        if let Some(x) = line.strip_prefix(tag.as_bytes()) {
+            let out = if x.is_empty() {
+                let mut buf = Vec::new();
+                grab(tag.as_bytes(), &mut buf, line, reader, need_read)?;
+                OutFile {
+                    name: tag[1..].to_string(),
+                    matcher: MatchMaker::make2("exact", std::str::from_utf8(&buf)?)?,
+                    content: buf,
+                }
+            } else {
+                let x = x.trimw_start();
+                OutFile {
+                    name: tag[1..].to_string(),
+                    matcher: MatchMaker::make(std::str::from_utf8(x)?)?,
+                    content: x.to_vec(),
+                }
+            };
+            if tag == "#stderr" {
+                self.stderr = out;
+            } else {
+                self.stdout = out;
+            }
+            Ok(true)
+        } else {
+            Ok(false)
+        }
+    }
     /// parse the file
     pub fn open(&mut self, file: &str) -> Result<()> {
         self.name = file.to_string();
@@ -106,33 +184,25 @@ impl Test {
             }
             let mut need_read = true;
             if let Some(x) = line.strip_prefix(b"#command") {
-                let y = skip_leading_white(x);
+                let y = x.trimw_start();
                 self.cmd = y.to_vec();
-            } else if grab(
-                b"#stdin",
-                &mut self.stdin,
-                &mut line,
-                &mut reader,
-                &mut need_read,
-            )? || grab(
-                b"#stdout",
-                &mut self.stdout,
-                &mut line,
-                &mut reader,
-                &mut need_read,
-            )? || grab(
-                b"#stderr",
-                &mut self.stderr,
-                &mut line,
-                &mut reader,
-                &mut need_read,
-            )? {
+            } else if self.add_outfile("#stdout", &mut line, &mut reader, &mut need_read)?
+                || self.add_outfile("#stderr", &mut line, &mut reader, &mut need_read)?
+                || grab(
+                    b"#stdin",
+                    &mut self.stdin,
+                    &mut line,
+                    &mut reader,
+                    &mut need_read,
+                )?
+            {
+                /* do nothing */
             } else if let Some(x) = line.strip_prefix(b"#status") {
-                let y = skip_leading_white(x);
+                let y = x.trimw_start();
                 self.code = str_to_i_whole(y)? as i32;
             } else if let Some(x) = line.strip_prefix(b"#infile") {
-                let y = skip_leading_white(x);
-                let mut f = OneFile {
+                let y = x.trimw_start();
+                let mut f = InFile {
                     name: String::from_utf8(y.to_vec())?,
                     ..Default::default()
                 };
@@ -140,18 +210,34 @@ impl Test {
                 grab(b"", &mut f.content, &mut line, &mut reader, &mut need_read)?;
                 self.in_files.push(f);
             } else if let Some(x) = line.strip_prefix(b"#outfile") {
-                let y = skip_leading_white(x);
-                let mut f = OneFile {
-                    name: String::from_utf8(y.to_vec())?,
-                    ..Default::default()
-                };
-                line.clear();
-                grab(b"", &mut f.content, &mut line, &mut reader, &mut need_read)?;
-                self.out_files.push(f);
+                let y = std::str::from_utf8(x)?;
+                let y = y.trimw_start();
+                if let Some((name, matcher)) = y.split_once(' ') {
+                    let f = OutFile {
+                        name: name.to_string(),
+                        content: matcher.as_bytes().to_vec(),
+                        matcher: MatchMaker::make(matcher)?,
+                    };
+                    self.out_files.push(f);
+                } else {
+                    let name = y.to_string();
+                    line.clear();
+                    let mut buf = Vec::new();
+                    grab(b"", &mut buf, &mut line, &mut reader, &mut need_read)?;
+                    let f = OutFile {
+                        name,
+                        matcher: MatchMaker::make2("exact", std::str::from_utf8(&buf)?)?,
+                        content: buf,
+                    };
+                    self.out_files.push(f);
+                }
             } else if line.starts_with(b"# ") {
                 // comment
             } else {
-                return err!("Unexpected line in test file : {:#?}", line);
+                return err!(
+                    "Unexpected line in test file : {}",
+                    std::str::from_utf8(&line)?
+                );
             }
             if need_read {
                 line.clear();
@@ -166,25 +252,34 @@ impl Test {
 
     /// run the test
     pub fn run(&mut self, config: &Config) -> Result<bool> {
-        let tmp = TempDir::new("tooltest")?;
+        let mut keep_files = false;
+        let mut tmp: String = if config.tmpdir.is_empty() {
+            config.tmp.path().to_owned().to_string_lossy().to_string()
+        } else {
+            keep_files = true;
+            config.tmpdir.clone()
+        };
+        if !tmp.ends_with('/') {
+            tmp.push('/');
+        }
         for x in &self.in_files {
-            let mut fname = tmp.path().to_owned();
-            fname.push(&x.name);
-            let mut w = get_writer(&fname.to_string_lossy())?;
+            let mut fname = tmp.clone();
+            fname.push_str(&x.name);
+            let mut w = get_writer(&fname)?;
             w.write_all(&x.content)?;
         }
         #[allow(clippy::redundant_clone)]
-        let mut tmp_stdin = tmp.path().to_owned();
-        tmp_stdin.push("stdin");
+        let mut tmp_stdin = tmp.clone();
+        tmp_stdin.push_str("stdin");
         {
             let mut w = std::fs::OpenOptions::new()
                 .write(true)
                 .create(true)
-                .open(&tmp_stdin.as_os_str())?;
+                .open(&tmp_stdin)?;
             w.write_all(&self.stdin)?;
         }
         let ncmd = String::from_utf8_lossy(&self.cmd)
-            .replace("$TMP", &tmp.path().display().to_string())
+            .replace("$TMP", &tmp)
             .as_bytes()
             .to_vec();
         //        prerr(&[b"About to run ", &ncmd]);
@@ -212,10 +307,10 @@ impl Test {
                 output = x;
             }
         }
-        let mut files = read_dir(tmp.path())?;
-        nuke(&mut files, &"stdin".to_string());
+        let mut files = read_dir(Path::new(&tmp))?;
+        nuke(&mut files, &"stdin".to_string(), &tmp, keep_files)?;
         for x in &self.in_files {
-            nuke(&mut files, &x.name);
+            nuke(&mut files, &x.name, &tmp, keep_files)?;
         }
         let mut failed = false;
         match output.status.code() {
@@ -232,29 +327,28 @@ impl Test {
         }
         //	eprintln!("Got input {} {} {}", self.code, self.stdout.len(), self.stderr.len());
         //	eprintln!("Got output {} {} {}", output.status.code().unwrap(), output.stdout.len(), output.stderr.len());
-        if output.stderr != self.stderr {
+        if !self.stderr.matcher.do_match_safe(&output.stderr) {
             failed = true;
             prerr(&[
                 b"Stderr was\n",
                 &output.stderr,
                 b"\ninstead of\n",
-                &self.stderr,
+                &self.stderr.content,
             ]);
         }
-        if output.stdout != self.stdout {
+        if !self.stdout.matcher.do_match_safe(&output.stdout) {
             failed = true;
             prerr(&[
                 b"Stdout was\n",
                 &output.stdout,
                 b"\ninstead of\n",
-                &self.stdout,
+                &self.stdout.content,
             ]);
         }
         for x in &self.out_files {
-            nuke(&mut files, &x.name);
-            let mut fname = tmp.path().to_owned();
-            fname.push(&x.name);
-            match get_reader(&fname.to_string_lossy()) {
+            let mut fname = tmp.clone();
+            fname.push_str(&x.name);
+            match get_reader(&fname) {
                 Err(e) => {
                     failed = true;
                     eprintln!("{:?}", e);
@@ -262,7 +356,7 @@ impl Test {
                 Ok(mut f) => {
                     let mut body = Vec::new();
                     f.read_to_end(&mut body)?;
-                    if body != x.content {
+                    if !x.matcher.do_match_safe(&body) {
                         failed = true;
                         prerr(&[
                             b"File ",
@@ -275,14 +369,26 @@ impl Test {
                     }
                 }
             }
+            nuke(&mut files, &x.name, &tmp, keep_files)?;
         }
-        if !files.is_empty() {
+        if !keep_files && !files.is_empty() {
             failed = true;
             eprintln!("Files left over in TMP dir :");
             for x in &files {
                 eprintln!("{}", x);
+                delete_file(x, &tmp)?;
             }
         }
+        if keep_files {
+            let mut fname = tmp.clone();
+            fname.push_str("stdout");
+            let mut w = get_writer(&fname)?;
+            w.write_all(&output.stdout)?;
+            fname = tmp.clone();
+            fname.push_str("stderr");
+            w = get_writer(&fname)?;
+            w.write_all(&output.stderr)?;
+        }
         if failed {
             prerr(&[b"Test ", self.name.as_bytes(), b" failed ", &self.cmd]);
         }
@@ -293,21 +399,34 @@ impl Test {
 
 /// Global config for running a bunch of tooltests
 /// Accumulates statistics
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct Config {
     pass: usize,
     fail: usize,
     bindir: String,
+    tmpdir: String,
+    tmp: TempDir,
 }
 
 impl Config {
     /// new
-    pub fn new() -> Self {
-        Self {
+    pub fn new() -> Result<Self> {
+        Ok(Self {
             pass: 0,
             fail: 0,
             bindir: "./".to_string(),
+            tmpdir: String::new(),
+            tmp: TempDir::new("tooltest")?,
+        })
+    }
+    /// set tmp dir
+    pub fn tmp(&mut self, path: &str) -> Result<()> {
+        self.tmpdir = path.to_string();
+        if !self.tmpdir.ends_with('/') {
+            self.tmpdir.push('/');
         }
+        fs::create_dir_all(&self.tmpdir)?;
+        Ok(())
     }
     /// set bin dir
     pub fn bin(&mut self, path: &str) {
@@ -330,21 +449,32 @@ impl Config {
             Ok(())
         }
     }
-    /// run
-    pub fn run(&mut self, file: &str) -> Result<bool> {
+    fn do_run(&mut self, file: &str) -> Result<bool> {
         let mut t = Test::new();
         t.open(file)?;
-        if t.run(self)? {
-            self.pass += 1;
-        } else {
-            self.fail += 1;
+        t.run(self)
+    }
+    /// run
+    pub fn run(&mut self, file: &str) {
+        match self.do_run(file) {
+            Ok(x) => {
+                if x {
+                    self.pass += 1;
+                } else {
+                    self.fail += 1;
+                }
+            }
+            Err(x) => {
+                self.fail += 1;
+                eprintln!("Failure : {}", x);
+                prerr(&[b"Test ", file.as_bytes(), b" failed "]);
+            }
         }
-        Ok(true)
     }
 }
 
 impl Default for Config {
     fn default() -> Self {
-        Self::new()
+        Self::new().unwrap()
     }
 }
